# Software License Agreement (BSD License)
#
# Copyright (c) 2012, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from python_qt_binding.QtCore import QEvent, QObject, Signal


class WindowChangedSignaler(QObject):
    """Signaler for for filtered events of an icon or title change of a widget."""

    hide_signal = Signal(object)
    show_signal = Signal(object)

    window_icon_changed_signal = Signal(object)
    window_title_changed_signal = Signal(object)

    def __init__(self, widget, parent=None):
        super(WindowChangedSignaler, self).__init__(parent)
        self._widget = widget
        self.setObjectName('WindowChangedSignaler__' + self._widget.objectName())
        self._recursive_invocation = False
        self._widget.installEventFilter(self)

    def emit_all(self):
        if self._widget.isVisible():
            self.show_signal.emit(self._widget)
        else:
            self.hide_signal.emit(self._widget)
        self.window_icon_changed_signal.emit(self._widget)
        self.window_title_changed_signal.emit(self._widget)

    def eventFilter(self, watched, event):
        if event.type() == QEvent.Hide:
            self.hide_signal.emit(watched)

        if event.type() == QEvent.Show:
            self.show_signal.emit(watched)

        if event.type() == QEvent.WindowIconChange:
            # prevent emitting recursive event when emitted signal will trigger another event
            if not self._recursive_invocation:
                self._recursive_invocation = True
                self.window_icon_changed_signal.emit(watched)
                self._recursive_invocation = False

        if event.type() == QEvent.WindowTitleChange:
            # prevent emitting recursive event when emitted signal will trigger another event
            if not self._recursive_invocation:
                self._recursive_invocation = True
                self.window_title_changed_signal.emit(watched)
                self._recursive_invocation = False

        return QObject.eventFilter(self, watched, event)
